==> cloud-5.1.1.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/pages/EntityCreatePage.tsx <==
import ErrorAlert from 'atoms/ErrorAlert';
import ENTITY_FORM_LIST from 'constant/templates/form/entity_form_template';
import useDrupalJsonApi, { GetJsonDataType } from 'hooks/drupal_jsonapi';
import useDrupalTranslation from 'hooks/drupal_translation';
import EntityFormRecord from 'model/EntityFormRecord';
import EntityFormTemplate from 'model/EntityFormTemplate';
import EntityFormLabel from 'molecules/EntityFormLabel';
import Breadcrumbs from 'organisms/Breadcrumbs';
import EntityFormPanel from 'organisms/EntityFormPanel';
import PageTitle from 'organisms/PageTitle';
import SideBar from 'organisms/SideBar';
import OpenStackHeatCreatePage from 'pages/openstack/heat/OpenStackHeatCreatePage';
import OpenStackHeatPreviewPage from 'pages/openstack/heat/OpenStackHeatPreviewPage';
import { useEffect, useState } from 'react';
import { Button, Col, Container, Form, Row } from 'react-bootstrap';
import { BsCheckLg } from 'react-icons/bs';
import { useHistory, useParams } from 'react-router-dom';
import { convertFormDataToSendData } from 'service/convert';

/**
 * Generate initial values for form input.
 */
const calcDefaultFormFata = async (
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
  getJsonData: GetJsonDataType
) => {
  /**
   * Create a list of key strings and default values.
   *
   * Note: Pretreatment to facilitate the second half of the process.
   */
  const formInfoList: { key: string, type: string, defaultValue: any }[] = [];
  for (const entityRecord of entityFormTemplate.entityRecords) {
    // Process only forms contained in Panel.
    if (entityRecord.type !== 'panel') {
      continue;
    }

    // Load the form inside each Panel.
    for (const keyValueRecord of entityRecord.keyValueRecords) {
      /**
       * In the case of a time input form,
       * the hour and minute are each given an initial value.
       */
      if (keyValueRecord.type === 'time') {
        formInfoList.push({ key: keyValueRecord.hourName, type: keyValueRecord.type, defaultValue: keyValueRecord.defaultValue[0] });
        formInfoList.push({ key: keyValueRecord.minuteName, type: keyValueRecord.type, defaultValue: keyValueRecord.defaultValue[1] });
        continue;
      }

      /**
       * The initial value is null in the case of a form for a file,
       * otherwise the branch is branched depending on
       * whether an initial value is given.
       */
      formInfoList.push({
        key: keyValueRecord.name,
        type: keyValueRecord.type,
        defaultValue: keyValueRecord.type === 'file'
          ? null
          : 'defaultValue' in keyValueRecord
            ? keyValueRecord.defaultValue
            : undefined
      });
    }
  }

  // Set initial values.
  const newFormData: Record<string, any> = {};
  for (const formInfo of formInfoList) {
    // Handling of types that are not entered in the form, such as 'cost' types.
    if (formInfo.defaultValue === undefined) {
      continue;
    }

    newFormData[formInfo.key] = formInfo.defaultValue;
  }

  /**
   * If the initial value is specified by URL,
   * the read operation is performed.
   */
  for (const entityRecord of entityFormTemplate.entityRecords) {
    // Process only forms contained in Panel.
    if (entityRecord.type !== 'panel') {
      continue;
    }

    /**
     * Check to see if the initial value is given by URL,
     * and if so, perform the load process.
     */
    for (const keyValueRecord of entityRecord.keyValueRecords) {
      if (keyValueRecord.type !== 'default'
        && keyValueRecord.type !== 'textarea'
        && keyValueRecord.type !== 'hidden'
        && keyValueRecord.type !== 'select') {
        continue;
      }
      if (keyValueRecord.defaultValueUrl === undefined) {
        continue;
      }
      const replacedUrl = keyValueRecord.defaultValueUrl.replace('{cloud_context}', cloudContext);
      if (replacedUrl.endsWith('/')) {
        continue;
      }
      const jsonData = await getJsonData<string>(replacedUrl);
      if (jsonData.length > 0) {
        newFormData[keyValueRecord.name] = jsonData;
      }
    }
  }

  return newFormData;
}

/**
 * Custom Hooks for form data.
 */
const useFormData = (
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
) => {

  const [formData, setFormData] = useState<Record<string, any>>({});
  const { getJsonData } = useDrupalJsonApi();

  // Initialize form data.
  useEffect(() => {
    calcDefaultFormFata(cloudContext, entityFormTemplate, getJsonData).then((defaultFormData) => {
      setFormData(defaultFormData);
    })
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  return {
    formData,
    setFormData
  };

}

/**
 * Custom Hooks for form template.
 */
const useFormTemplate = (action: string) => {

  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();

  const entityFormTemplateList = ENTITY_FORM_LIST.filter((template) => {
    return template.cloudServiceProvider === params.cloudServiceProvider
      && (template.entityName === params.entityName || template.entityName === '')
      && template.actionType === action
  });

  return {
    formTemplate: entityFormTemplateList.length >= 1
      ? entityFormTemplateList[0] : undefined
  };

}

/**
 * Default page if form data cannot be retrieved.
 */
const DefaultPage = () => {

  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();

  return <>
    <PageTitle />
    <Breadcrumbs />
    <SideBar />
    <Container fluid className="px-0">
      <Row>
        <Col>
          <span>EntityCreatePage</span><br />
          <span>cloudServiceProvider={params.cloudServiceProvider}</span><br />
          <span>cloudContext={params.cloudContext}</span><br />
          <span>entityName={params.entityName}</span><br />
          <span>action=create</span>
        </Col>
      </Row>
    </Container>
  </>;

}

/**
 * A block of forms to be displayed based on the form template.
 */
const EntityRecordForm = ({
  cloudServiceProvider,
  cloudContext,
  entityName,
  action,
  formData,
  setFormData,
  entityRecord,
}: {
  cloudServiceProvider: string,
  cloudContext: string,
  entityName: string,
  action: string,
  formData: Record<string, any>,
  setFormData: (f: Record<string, any>) => void,
  entityRecord: EntityFormRecord,
}) => {

  switch (entityRecord.type) {
    case 'panel':
      return <EntityFormPanel
        cloudServiceProvider={cloudServiceProvider}
        cloudContext={cloudContext}
        entityName={entityName}
        actionType={action}
        entityRecord={entityRecord}
        formData={formData}
        setFormData={setFormData} />;
    case 'label':
      return <EntityFormLabel label={entityRecord.text} entityName={entityName} />;
    default:
      return <></>;
  };

}

/**
 * Page of the form to create an entity.
 *
 * @param cloudContext The cloud context.
 * @param entityFormTemplate The form template for entity.
 * @param entityName The entity type.
 * @param action The CxUD action type.
 */
const EntityCreatePageImpl = ({ cloudContext, entityFormTemplate, entityName, action }: {
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
  entityName: string,
  action: string,
}) => {

  const { formData, setFormData } = useFormData(cloudContext, entityFormTemplate);
  const [errorMessage, setErrorMessage] = useState('');
  const history = useHistory();
  const { t } = useDrupalTranslation();

  useEffect(() => {
    console.group('FormData');
    console.log(formData);
    console.groupEnd();
  }, [formData]);

  /**
   * Save entity's data.
   */
  const save = async () => {
    const url = `/cloud_dashboard/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityFormTemplate.cloudServiceProvider}_${entityName}/${action}`;

    const response = await fetch(url, {
      method: 'POST',
      body: convertFormDataToSendData(formData, entityFormTemplate),
    });
    if (!response.ok) {
      console.group('Create Request');
      console.log('Result: Failed');
      console.log('Response:');
      console.error(await response.json());
      console.groupEnd();
      setErrorMessage('Create failed.');
    } else {
      console.group('Create Request');
      console.log('Result: Success');
      console.log('Response:');
      console.log(await response.json());
      console.groupEnd();
      history.push(`/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityName}`);
    }
  }

  return <>
    <PageTitle action={action} />
    <Breadcrumbs action={action} />
    <SideBar />
    <Container fluid className="px-0">
      <Row className="mx-0">
        <Col>
          <Form>
            <ErrorAlert message={errorMessage} />
            {
              entityFormTemplate.entityRecords.map((entityRecord, index) => {
                return <EntityRecordForm key={index}
                  cloudServiceProvider={entityFormTemplate.cloudServiceProvider}
                  cloudContext={cloudContext}
                  entityName={entityName}
                  action={entityFormTemplate.actionType}
                  formData={formData}
                  setFormData={setFormData}
                  entityRecord={entityRecord} />;
              })
            }
            <Button className="ripple-effect" onClick={save}>
              <BsCheckLg />{' ' + entityFormTemplate.submitButtonLabel !== undefined
                ? t(entityFormTemplate.submitButtonLabel ?? 'Save')
                : Drupal.t('Save')
              }
            </Button>
          </Form>
        </Col>
      </Row>
    </Container>
  </>;

}

/**
 * Page of the form to create an entity.
 *
 * @param action The CxUD action type.
 */
const EntityCreatePage = ({ action }: {
  action: string
}) => {

  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();
  const { formTemplate } = useFormTemplate(action);

  if (!formTemplate) {
    return <DefaultPage />;
  }

  if (formTemplate.cloudServiceProvider === 'openstack'
    && formTemplate.entityName === 'stack'
    && formTemplate.actionType === 'create') {
    return <OpenStackHeatCreatePage />;
  }

  if (formTemplate.cloudServiceProvider === 'openstack'
    && formTemplate.entityName === 'stack'
    && formTemplate.actionType === 'preview') {
    return <OpenStackHeatPreviewPage />;
  }

  return <EntityCreatePageImpl
    cloudContext={params.cloudContext}
    entityFormTemplate={formTemplate}
    entityName={params.entityName}
    action={action} />;

}

export default EntityCreatePage;

==> cloud-5.2.0.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/pages/EntityCreatePage.tsx <==
import ErrorAlert from 'atoms/ErrorAlert';
import LabelText from 'atoms/LabelText';
import ENTITY_FORM_LIST from 'constant/templates/form/entity_form_template';
import useDrupalJsonApi, { GetJsonDataType } from 'hooks/drupal_jsonapi';
import useDrupalTranslation from 'hooks/drupal_translation';
import EntityFormRecord from 'model/EntityFormRecord';
import EntityFormTemplate from 'model/EntityFormTemplate';
import EntityFormLabel from 'molecules/EntityFormLabel';
import Breadcrumbs from 'organisms/Breadcrumbs';
import EntityFormPanel from 'organisms/EntityFormPanel';
import PageTitle from 'organisms/PageTitle';
import SideBar from 'organisms/SideBar';
import OpenStackHeatCreatePage from 'pages/openstack/heat/OpenStackHeatCreatePage';
import OpenStackHeatPreviewPage from 'pages/openstack/heat/OpenStackHeatPreviewPage';
import { useEffect, useState } from 'react';
import { Button, Col, Container, Form, Row } from 'react-bootstrap';
import { useHistory, useParams } from 'react-router-dom';
import { convertFormDataToSendData } from 'service/convert';

/**
 * Generate initial values for form input.
 */
const calcDefaultFormFata = async (
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
  getJsonData: GetJsonDataType
) => {
  /**
   * Create a list of key strings and default values.
   *
   * Note: Pretreatment to facilitate the second half of the process.
   */
  const formInfoList: { key: string, type: string, defaultValue: any }[] = [];
  for (const entityRecord of entityFormTemplate.entityRecords) {
    // Process only forms contained in Panel.
    if (entityRecord.type !== 'panel') {
      continue;
    }

    // Load the form inside each Panel.
    for (const keyValueRecord of entityRecord.keyValueRecords) {
      /**
       * In the case of a time input form,
       * the hour and minute are each given an initial value.
       */
      if (keyValueRecord.type === 'time') {
        formInfoList.push({ key: keyValueRecord.hourName, type: keyValueRecord.type, defaultValue: keyValueRecord.defaultValue[0] });
        formInfoList.push({ key: keyValueRecord.minuteName, type: keyValueRecord.type, defaultValue: keyValueRecord.defaultValue[1] });
        continue;
      }

      /**
       * The initial value is null in the case of a form for a file,
       * otherwise the branch is branched depending on
       * whether an initial value is given.
       */
      formInfoList.push({
        key: keyValueRecord.name,
        type: keyValueRecord.type,
        defaultValue: keyValueRecord.type === 'file'
          ? null
          : 'defaultValue' in keyValueRecord
            ? keyValueRecord.defaultValue
            : undefined
      });
    }
  }

  // Set initial values.
  const newFormData: Record<string, any> = {};
  for (const formInfo of formInfoList) {
    // Handling of types that are not entered in the form, such as 'cost' types.
    if (formInfo.defaultValue === undefined) {
      continue;
    }

    newFormData[formInfo.key] = formInfo.defaultValue;
  }

  /**
   * If the initial value is specified by URL,
   * the read operation is performed.
   */
  for (const entityRecord of entityFormTemplate.entityRecords) {
    // Process only forms contained in Panel.
    if (entityRecord.type !== 'panel') {
      continue;
    }

    /**
     * Check to see if the initial value is given by URL,
     * and if so, perform the load process.
     */
    for (const keyValueRecord of entityRecord.keyValueRecords) {
      if (keyValueRecord.type !== 'default'
        && keyValueRecord.type !== 'textarea'
        && keyValueRecord.type !== 'hidden'
        && keyValueRecord.type !== 'select') {
        continue;
      }
      if (keyValueRecord.defaultValueUrl === undefined) {
        continue;
      }
      const replacedUrl = keyValueRecord.defaultValueUrl.replaceAll('{cloud_context}', cloudContext);
      if (replacedUrl.endsWith('/')) {
        continue;
      }
      const jsonData = await getJsonData<string>(replacedUrl);
      if (jsonData.length > 0) {
        newFormData[keyValueRecord.name] = jsonData;
      }
    }
  }

  return newFormData;
}

/**
 * Custom Hooks for form data.
 */
const useFormData = (
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
) => {

  const [formData, setFormData] = useState<Record<string, any>>({});
  const { getJsonData } = useDrupalJsonApi();

  // Initialize form data.
  useEffect(() => {
    calcDefaultFormFata(cloudContext, entityFormTemplate, getJsonData).then((defaultFormData) => {
      setFormData(defaultFormData);
    })
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  return {
    formData,
    setFormData
  };

}

/**
 * Custom Hooks for form template.
 */
const useFormTemplate = (action: string) => {

  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();

  const entityFormTemplateList = ENTITY_FORM_LIST.filter((template) => {
    return template.cloudServiceProvider === params.cloudServiceProvider
      && (template.entityName === params.entityName || template.entityName === '')
      && template.actionType === action
  });

  return {
    formTemplate: entityFormTemplateList.length >= 1
      ? entityFormTemplateList[0] : undefined
  };

}

/**
 * Default page if form data cannot be retrieved.
 */
const DefaultPage = () => {

  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();

  return <>
    <PageTitle />
    <Breadcrumbs />
    <SideBar />
    <Container fluid className="px-0">
      <Row>
        <Col>
          <span>EntityCreatePage</span><br />
          <span>cloudServiceProvider={params.cloudServiceProvider}</span><br />
          <span>cloudContext={params.cloudContext}</span><br />
          <span>entityName={params.entityName}</span><br />
          <span>action=create</span>
        </Col>
      </Row>
    </Container>
  </>;

}

/**
 * A block of forms to be displayed based on the form template.
 */
const EntityRecordForm = ({
  cloudServiceProvider,
  cloudContext,
  entityName,
  action,
  formData,
  setFormData,
  entityRecord,
}: {
  cloudServiceProvider: string,
  cloudContext: string,
  entityName: string,
  action: string,
  formData: Record<string, any>,
  setFormData: (f: Record<string, any>) => void,
  entityRecord: EntityFormRecord,
}) => {

  switch (entityRecord.type) {
    case 'panel':
      return <EntityFormPanel
        cloudServiceProvider={cloudServiceProvider}
        cloudContext={cloudContext}
        entityName={entityName}
        actionType={action}
        entityRecord={entityRecord}
        formData={formData}
        setFormData={setFormData} />;
    case 'label':
      return <EntityFormLabel label={entityRecord.text} entityName={entityName} />;
    default:
      return <></>;
  };

}

/**
 * Page of the form to create an entity.
 *
 * @param cloudContext The cloud context.
 * @param entityFormTemplate The form template for entity.
 * @param entityName The entity type.
 * @param action The CxUD action type.
 */
const EntityCreatePageImpl = ({ cloudContext, entityFormTemplate, entityName, action }: {
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
  entityName: string,
  action: string,
}) => {

  const { formData, setFormData } = useFormData(cloudContext, entityFormTemplate);
  const [errorMessage, setErrorMessage] = useState('');
  const history = useHistory();
  const { t } = useDrupalTranslation();

  useEffect(() => {
    console.group('FormData');
    console.log(formData);
    console.groupEnd();
  }, [formData]);

  /**
   * Save entity's data.
   */
  const save = async () => {
    const url = `/cloud_dashboard/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityFormTemplate.cloudServiceProvider}_${entityName}/${action}`;

    const response = await fetch(url, {
      method: 'POST',
      body: convertFormDataToSendData(formData, entityFormTemplate),
    });
    if (!response.ok) {
      console.group('Create Request');
      console.log('Result: Failed');
      console.log('Response:');
      console.error(await response.json());
      console.groupEnd();
      setErrorMessage('Create failed.');
    } else {
      console.group('Create Request');
      console.log('Result: Success');
      console.log('Response:');
      console.log(await response.json());
      console.groupEnd();
      history.push(`/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityName}`);
    }
  }

  const saveButtonLabel = entityFormTemplate.submitButtonLabel !== undefined
    ? (entityFormTemplate.submitButtonLabel ?? 'Save')
    : 'Save';

  return <>
    <PageTitle action={action} />
    <Breadcrumbs action={action} />
    <SideBar />
    <Container fluid className="px-0">
      <Row className="mx-0">
        <Col>
          <Form>
            <ErrorAlert message={errorMessage} />
            {
              entityFormTemplate.entityRecords.map((entityRecord, index) => {
                return <EntityRecordForm key={index}
                  cloudServiceProvider={entityFormTemplate.cloudServiceProvider}
                  cloudContext={cloudContext}
                  entityName={entityName}
                  action={entityFormTemplate.actionType}
                  formData={formData}
                  setFormData={setFormData}
                  entityRecord={entityRecord} />;
              })
            }
            <Button className="ripple-effect" onClick={save}>
              <LabelText text={saveButtonLabel} />
            </Button>
          </Form>
        </Col>
      </Row>
    </Container>
  </>;

}

/**
 * Page of the form to create an entity.
 *
 * @param action The CxUD action type.
 */
const EntityCreatePage = ({ action }: {
  action: string
}) => {

  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();
  const { formTemplate } = useFormTemplate(action);

  if (!formTemplate) {
    return <DefaultPage />;
  }

  if (formTemplate.cloudServiceProvider === 'openstack'
    && formTemplate.entityName === 'stack'
    && formTemplate.actionType === 'create') {
    return <OpenStackHeatCreatePage />;
  }

  if (formTemplate.cloudServiceProvider === 'openstack'
    && formTemplate.entityName === 'stack'
    && formTemplate.actionType === 'preview') {
    return <OpenStackHeatPreviewPage />;
  }

  return <EntityCreatePageImpl
    cloudContext={params.cloudContext}
    entityFormTemplate={formTemplate}
    entityName={params.entityName}
    action={action} />;

}

export default EntityCreatePage;
